/* Copyright (c) 2001 TANAKA Kei <peg00625@nifty.com>
   References: OpenBloskS Users ML [obu:00209] and RFC 1321, RFC 1939
   gcc -O2 -o pop3led pop3led.c
   gcc -O2 -DAPOP -o pop3led.APOP pop3led.c md5c.c
   gcc -O2 -DAPOP -DSYSLOG -o pop3led.SYSLOG pop3led.c md5c.c */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <netdb.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>

#ifdef SYSLOG
#include <syslog.h>
#define PRIORITY	(LOG_USER|LOG_INFO)
#endif

#ifdef APOP
#include "global.h"	/* RFC 1321 */
#include "md5.h"	/* RFC 1321 */
#undef APOP

/* phases */
#define APOP	0
#endif
#define USER	1
#define PASS	2
#define STAT	3
#define	STATre	4
#define QUIT	5

int pop3stat(char *server, char *user, char *pass)
{
	struct servent *service;
	struct hostent *host;
	struct protoent *protocol;
	struct in_addr addr;
	struct sockaddr_in serv_addr;
	struct timeval timeout;
	fd_set fds;
	char buf[512];
	int byte, sockfd;
	int re = -1;

#ifdef APOP
	/* RFC 1321 */
	MD5_CTX MD5context;
	unsigned char MD5digest[16];
	char *MD5string;

	int phase = APOP;

	if (*user == '+') user++;	/* APOP */
	else phase++;	/* Not APOP */
#else
	int phase = USER;
#endif

	/* get Service Entry (POP3/TCP) from /etc/services */
	if ((service = getservbyname("pop-3", "tcp")) == NULL) {
#ifdef SYSLOG
		syslog(PRIORITY, "getservbyname(\"pop-3\", \"tcp\") failed");
#endif
		return re;
	}

	/* get Host Entry (POP3 Server) from DNS, /etc/hosts, NIS or YP */
	if ((host = gethostbyname(server)) == NULL) {	/* server as hostname */
		if (inet_aton(server, &addr) == 0) {
#ifdef SYSLOG
			syslog(PRIORITY, "%s: inet_aton() failed", server);
#endif
			return re;
		}
		if ((host = gethostbyaddr((char *)&addr.s_addr,
		sizeof(addr.s_addr), AF_INET)) == NULL) {	/* server as IP address */
#ifdef SYSLOG
			syslog(PRIORITY, "%s: gethostbyaddr() failed", server);
#endif
			return re;
		}
	}

	/* get Protocol Entry (IP) from /etc/protocols */
	if ((protocol = getprotobyname("ip")) == NULL) {
#ifdef SYSLOG
		syslog(PRIORITY, "getprotobyname(\"ip\") failed");
#endif
		return re;
	}

	/* socket(Protocol Family (IPv4), TCP, Protocol number (IP)) */
	if ((sockfd = socket(PF_INET, SOCK_STREAM, protocol->p_proto)) == -1) {
#ifdef SYSLOG
		syslog(PRIORITY, "%s: socket() failed", server);
#endif
		return re;
	}

	memset(&serv_addr, 0, sizeof(serv_addr));
	serv_addr.sin_family = AF_INET;		/* Address Family (IPv4) */
	serv_addr.sin_port = service->s_port;	/* Port number */
	memcpy(&serv_addr.sin_addr, host->h_addr_list[0], host->h_length);
	if (connect(sockfd, &serv_addr, sizeof(serv_addr)) == -1) {
#ifdef SYSLOG
		syslog(PRIORITY, "%s: connect() failed", server);
#endif
		return re;
	}

	FD_ZERO(&fds);
	FD_SET(sockfd, &fds);

	for (;;) {
		timeout.tv_sec = 300;
		timeout.tv_usec = 0;
		if (select(sockfd+1, &fds, NULL, NULL, &timeout) == -1) {
#ifdef SYSLOG
			syslog(PRIORITY, "select: %m");
#endif
			break;
		}
		if (FD_ISSET(sockfd, &fds)) {
			byte = recv(sockfd, buf, sizeof(buf), 0);
			if (byte <= 0) break;
			buf[byte] = '\0';
#ifdef DEBUG
			printf("%s", buf);
#endif
			if (strncmp(buf, "+OK", 3)) {
#ifdef SYSLOG
				syslog(PRIORITY, "%s: %s", user, buf);
#endif
				phase = QUIT;
				if (strncmp(buf, "-ERR", 4)) phase++;
			}

			switch (phase) {
#ifdef APOP
			case APOP:
				MD5string = strchr(buf+3, '<');
				*(strchr(MD5string, '>')+1) = '\0';
				strcat(MD5string, pass);
				MD5Init(&MD5context);
				MD5Update(&MD5context, MD5string, strlen(MD5string));
				MD5Final(MD5digest, &MD5context);

				sprintf(buf, "APOP %s ", user);
				for (byte = 0; byte < 16; byte++) {
					sprintf(buf, "%s%02x", buf, MD5digest[byte]);
				}
				sprintf(buf, "%s\r\n", buf);
				phase = PASS;
				break;
#endif
			case USER:
				sprintf(buf, "USER %s\r\n", user);
				break;

			case PASS:
				sprintf(buf, "PASS %s\r\n", pass);
				break;

			case STAT:
				sprintf(buf, "STAT\r\n");
				break;

			case STATre:
				re = atoi(buf+3);
				phase++;

			case QUIT:
				sprintf(buf, "QUIT\r\n");
				break;

			default:
				close(sockfd);
				return re;
			}
#ifdef DEBUG
			printf("%s", buf);
#endif
			send(sockfd, buf, strlen(buf), 0);
			phase++;
		}
	}
	close(sockfd);
	return re;
}

void led1sec(int fd, char num)
{
	char led[2];
	char dot[] = { '-', '+' };	/* '-'=off, '+'=on */
	static int dotsw = 0;	/* off */

	led[0] = num;
	led[1] = dot[dotsw];
	dotsw ^= 1;		/* off->on, on->off */
	write(fd, led, 2);
#ifdef DEBUG
	printf("[%c%c]\n", led[0], led[1]);
#endif
	sleep(1);
}

int main(int argc, char **argv)
{
	int sec = 900;
	int timer;
	int fd, entry, num, total;
	int messages[6];
	char server[6][128], user[6][40], pass[6][40];

	if (argc > 1) {
		sec = atoi(*++argv);
		if (sec < 1) sec = 900;
	}

	if ((fd = open("/dev/segled", O_WRONLY)) == -1) {
		perror("open(\"/dev/segled\", O_WRONLY)");
		return 1;
	}

	for (entry = 0; entry < 9; entry++) {
		num = scanf("%s%s%s", server[entry], user[entry], pass[entry]);
		if (num == EOF || num < 3) break;
#ifdef DEBUG
		printf("%s@%s: %s\n", user[entry], server[entry], pass[entry]);
#endif
	}
	num = entry;
	for (;;) {
		led1sec(fd, '0');
		total = 0;
		for (entry = 0; entry < num; entry++) {
			messages[entry] = pop3stat(server[entry], user[entry], pass[entry]);
#ifdef DEBUG
			printf("[%c] %s@%s: %d\n", 'A'+entry,
			user[entry], server[entry], messages[entry]);
#endif
			if (messages[entry] > 0) {
				total += messages[entry];
				if (messages[entry] > 9) messages[entry] = 9;
			}
		}
		timer = sec;
		do {
			if (total) {
				for (entry = 0;  entry < num; entry++) {
					if (messages[entry] > 0) {
						led1sec(fd, 'A'+entry);
						led1sec(fd, '0'+messages[entry]);
						timer -= 2;
					}
				}
			}
			else {
				led1sec(fd, ' ');
				timer--;
			}
#ifdef DEBUG
			printf("%d\n", timer);
#endif
		} while (timer);
	}
}
